@@ -1,5 +1,5 @@
 /*
-    $Id: watchquagga.c,v 1.12 2007/05/10 02:38:52 paul Exp $
+    $Id$
 
     Monitor status of quagga daemons and restart if necessary.
 
@@ -1343,7 +1343,11 @@ main(int argc, char **argv)
   if (daemon_mode)
     {
       zlog_set_level(NULL, ZLOG_DEST_SYSLOG, MIN(gs.loglevel,LOG_DEBUG));
-      daemon(0, 0);
+      if (daemon (0, 0) < 0)
+	{
+	  fprintf(stderr, "Watchquagga daemon failed: %s", strerror(errno));
+	  exit (1);
+	}
     }
   else
     zlog_set_level(NULL, ZLOG_DEST_STDOUT, MIN(gs.loglevel,LOG_DEBUG));
