@@ -964,10 +964,10 @@ zread_ipv6_add (struct zserv *client, u_short length)
     api.metric = 0;
     
   if (IN6_IS_ADDR_UNSPECIFIED (&nexthop))
-    rib_add_ipv6 (api.type, api.flags, &p, NULL, ifindex, 0, api.metric,
+    rib_add_ipv6 (api.type, api.flags, &p, NULL, ifindex, zebrad.rtm_table_default, api.metric,
 		  api.distance);
   else
-    rib_add_ipv6 (api.type, api.flags, &p, &nexthop, ifindex, 0, api.metric,
+    rib_add_ipv6 (api.type, api.flags, &p, &nexthop, ifindex, zebrad.rtm_table_default, api.metric,
 		  api.distance);
   return 0;
 }
@@ -1030,9 +1030,9 @@ zread_ipv6_delete (struct zserv *client, u_short length)
     api.metric = 0;
     
   if (IN6_IS_ADDR_UNSPECIFIED (&nexthop))
-    rib_delete_ipv6 (api.type, api.flags, &p, NULL, ifindex, 0);
+    rib_delete_ipv6 (api.type, api.flags, &p, NULL, ifindex, client->rtm_table);
   else
-    rib_delete_ipv6 (api.type, api.flags, &p, &nexthop, ifindex, 0);
+    rib_delete_ipv6 (api.type, api.flags, &p, &nexthop, ifindex, client->rtm_table);
   return 0;
 }
 
@@ -1569,7 +1569,7 @@ config_write_table (struct vty *vty)
 }
 
 /* table node for routing tables. */
-struct cmd_node table_node =
+static struct cmd_node table_node =
 {
   TABLE_NODE,
   "",				/* This node has no interface. */
@@ -1689,7 +1689,7 @@ config_write_forwarding (struct vty *vty)
 }
 
 /* table node for routing tables. */
-struct cmd_node forwarding_node =
+static struct cmd_node forwarding_node =
 {
   FORWARDING_NODE,
   "",				/* This node has no interface. */
@@ -1704,13 +1704,6 @@ zebra_init (void)
   /* Client list init. */
   zebrad.client_list = list_new ();
 
-  /* Make zebra server socket. */
-#ifdef HAVE_TCP_ZEBRA
-  zebra_serv ();
-#else
-  zebra_serv_un (ZEBRA_SERV_PATH);
-#endif /* HAVE_TCP_ZEBRA */
-
   /* Install configuration write function. */
   install_node (&table_node, config_write_table);
   install_node (&forwarding_node, config_write_forwarding);
@@ -1737,3 +1730,14 @@ zebra_init (void)
   /* Route-map */
   zebra_route_map_init ();
 }
+
+/* Make zebra server socket, wiping any existing one (see bug #403). */
+void
+zebra_zserv_socket_init (void)
+{
+#ifdef HAVE_TCP_ZEBRA
+  zebra_serv ();
+#else
+  zebra_serv_un (ZEBRA_SERV_PATH);
+#endif /* HAVE_TCP_ZEBRA */
+}
